import pygame, random

from pykitten import utils
from pykitten.data import *
from pykitten.gui import ClickArea, ImgButton

from stuff import *
from unitdat import *
from maputils import *
from troops import *
from widgets import HitMaskButton


class Region(HitMaskButton):
    def __init__(self, name, owner, pos, map, id):
        HitMaskButton.__init__(self, HEX_img[owner], callback=self.clicked)
        self.id = id
        troops = {
            "Soldier":[],
            "Flamethrower":[],
            "Grenadier":[],
            "Tank":[],
            "Jet":[],
        }

        self.troops = troops.copy()
        self.build = troops.copy() #TODO: add P.U.-increasing buildings
        self.mpos = pos  #position on map
        self.new_troops = None
        self.map = map
        map_pos = self.map.pos
        self.real_pos = (pos[0]+map_pos[0], pos[1]+map_pos[1]) #pos on screen
        self.name = name
        self.adjacent = []

        self.res = (0,0,0) #(gold, metal, oil) given per turn
        self.pu = 2
        self.hq = False
        self.town = False
        self.owner = owner
        self.active = False
        self.moved = False

    def clicked(self):
        self.map.select(self.id)
        SFX["mapclick"].play()

    def highlight(self, play_snd=True):
        self.gen_image()
        if play_snd:
            SFX["maphover"].play()

    def unhighlight(self):
        if not self.active:
            self.gen_image()

    def set_owner(self, player="grey"):
        self.map.changed = 1
        self.owner = player
        self.gen_image()
        self.map.game.conti.update_territories()

    def add_neighbour(self, region):
        self.adjacent.append(region)

    def add_unit(self, type):
        unit = type(self.owner)
        self.troops[unit.type].append(unit)

    def add_new_troops(self):
        for nr in xrange(len(self.new_troops)):
            unit = UNIT_CLASSES[nr]
            for i in xrange(self.new_troops[nr]):
                self.add_unit(unit)
        self.new_troops = []

    def get_available(self):
        avail = {}
        for type in TYPES:
            avail[type] = []
            for t in self.troops[type]:
                if not t.moved:
                    avail[type].append(t)
        return avail

    def add_troops(self, troops):
        if not self.occupied():
            self.owner = self.map.game.player.color

        for type in troops:
            self.troops[type].extend(troops[type])


        self.gen_image()

    def rem_troops(self, troops={}):
        if not troops:  #rem all troops
            for type in self.troops.keys():
                self.troops[type] = []

        for type in troops: #rem specified units
            for unit in troops[type]:
                self.troops[type].remove(unit)

        self.gen_image()

    def gen_image(self):
        self.map.changed = 1

        name = self.owner
        if self.hover or self.active:
            name += "h"
        self.blit(HEX_img[name], (0,0))

        if self.hq:
            self.blit(HEX_img["hq"], (41,35))
        elif self.town:
            self.blit(HEX_img["town"], (41,35))

        if self.occupied():
            drawme = self.map.game.player.color==self.owner
            if not drawme:
                for id in self.adjacent:
                    if self.map.region[id].owner==self.map.game.player.color:
                        drawme = True
            if drawme:
#               x, y = utils.center(HEX_img["unit"].get_size(),self.get_size())
                self.blit(HEX_img["unit"], (34, 1))

        self.map.blit(self, self.mpos)

    def occupied(self):
        for type in self.troops.keys():
            if self.troops[type]:
                return True
        return False

    def set_attr(self, res, town, hq, pu):
        self.pu = pu
        self.hq = hq
        self.res = res
        self.town = town


class Map(pygame.Surface):
    def __init__(self, game, map=None, savegame=False):
        self.game = game

        if savegame:
            #load map from savegame
            self.load_savegame(savegame)
            return

        img = MAP_img[map.imgname]
        self.bgname = map.imgname

        map = map.map
        self.img=img
        pygame.Surface.__init__(self, img.get_size())
        self.game = game
        self.fill((255,0,255))
        self.set_colorkey((255,0,255))

        self.blit(img, (0,0))
        self.pos = (31,36)
        self.region = {}
        self.hq = {}
        self.load_regions(map)

        self.clicked = False
        self.changed = 1
        self.selected = None

        for r in self.region.keys():
            reg = self.region[r]
            pos = (reg.mpos[0]+self.pos[0], reg.mpos[1]+self.pos[1])
            reg.set_pos(reg.real_pos)
            self.blit(reg, reg.mpos)

    def redraw(self):
        for name in self.region.keys():
            self.region[name].gen_image()

    def select(self, regname):
        if self.game.from_reg:
            if not self.game.set_dest(dest=self.region[regname]):
                self.game.from_reg = None
                self.game.cur_troops = None

        if self.selected:
            if self.selected is self.region[regname]:
                return
            #deselect previously selected region
            self.selected.active = False
            self.selected.unhighlight()

        self.clicked = True
        reg = self.region[regname]
        reg.active = True
        reg.highlight(play_snd=False)
        self.changed = 1
        self.selected = reg

    def load_regions(self, map):
        r = []
        r.extend(REGNAMES)
        random.shuffle(r)

        nmap = calc_neighbours(map)

        self.region = {}
        for type in map.keys():
            for pos in map[type]:
                #standard values
                id = pos
                name = r.pop()
                hq = False
                owner = "grey"
                town = False
                pu = 0
                res = (0,0,0)

                if type=="town":
                    town = True
                    owner = "grey"
                    pu = random.randint(1,3)
                    pu *= 100
                    gold = random.randint(3,5)
                    metal = random.randint(1,3)
                    oil = random.randint(1,1)
                    res = (gold, metal, oil)
                elif type=="grey":
                    gold = random.randint(1,3)
                    metal = random.randint(1,5)
                    oil = random.randint(1,3)
                    res = (gold, metal, oil)
                else:
                    owner = type  #setup hq
                    pu = 200
                    hq = True
                    self.hq[owner] = id
                    res = (4,2,1)

                self.region[id] = Region(name, owner, pos, self, id)
                self.region[id].set_attr(res, town, hq, pu)
                self.region[id].add_unit(Soldier)

                for n in nmap[pos]:
                    self.region[id].add_neighbour(n)

    def load_savegame(self, game_data):
        #map bg image
        self.bgname = game_data["map_bg"]
        img = MAP_img[self.bgname]
        self.img=img
        pygame.Surface.__init__(self, img.get_size())
        self.fill((255,0,255))
        self.set_colorkey((255,0,255))
        self.blit(img, (0,0))
        self.pos = (31,36)
        self.region = {}

        self.clicked = False
        self.changed = 1
        self.selected = None

        #aquire map data
        self.hq = game_data["hq"]

        self.region = {}
        for id in game_data["regions"]:
            regdat = game_data["regions"][id]
            reg = Region(regdat.name, regdat.owner, regdat.pos,self,regdat.id)

            reg.troops = regdat.troops
            reg.new_troops = regdat.new_troops
            reg.build = regdat.build
            reg.adjacent = regdat.adjacent
            reg.res = regdat.res #(gold, metal, oil) given per turn
            reg.pu = regdat.pu
            reg.hq = regdat.hq
            reg.town = regdat.town
            reg.mpos = regdat.mpos
            reg.real_pos = regdat.real_pos

            self.region[id] = reg

        for r in self.region.keys():
            reg = self.region[r]
            pos = (reg.mpos[0]+self.pos[0], reg.mpos[1]+self.pos[1])
            reg.set_pos(reg.real_pos)
            self.blit(reg, reg.mpos)
