import pygame
from pygame.locals import *

from pykitten.kittens import Kitten
from pykitten import utils
from pykitten.engine import State

from stuff import *
from widgets import FlipButton

class BaseState(State):
    def __init__(self):
        State.__init__(self)
        self.screen = pygame.display.get_surface()
        self.bg = pygame.Surface((800,600))
        self.button = {}

    def add_button(self, button, pos, id=None):
        button.set_pos(pos)
        button.draw(self.bg)
        if id is not None:
            self.button[id] = button
        self.widgets.append(button)

class PopUpMenu(BaseState):
    def __init__(self, screen, window, pos=None):
        BaseState.__init__(self)
        self.bg = screen.copy()
        if pos is None:
            pos = utils.center(window.get_size(), (800,600))
        self.bg.blit(window, pos)
        self.cur_troops = None

        self.add_kbevent(KEYUP, K_ESCAPE, callback=self.abort)

    def abort(self):
        self.cur_troops=None
        self.quit()

    def main_start(self):
        self.screen.blit(self.bg, (0,0))
        pygame.display.flip()



class FadeState(BaseState):
    def __init__(self, add=1):
        BaseState.__init__(self)
        self.alpha = 0
        self.fading = 1
        self.fps = 25
        self.choice = None
        self.changed = 0
        self.add = add
        
        
    def update_screen(self):
        if not self.fading:
            for id in self.button:
                b = self.button[id]
                b.update()
                if b.changed:
                    b.changed = 0
                    self.changed = 1
                b.draw(self.bg)
            if self.changed:
                self.changed = 0
                self.screen.blit(self.bg, (0,0))
                pygame.display.flip()

        elif self.fading > 0:
            self.alpha += self.fading
            self.bg.set_alpha(self.alpha)
            self.fading += self.add
            if self.alpha > 200:
                self.fading = 0
                self.alpha = 255
                self.bg.set_alpha(255)
                self.fps = 20
                self.after_fadein()
            self.screen.blit(self.bg, (0,0))
            pygame.display.flip()
        
        elif self.fading < 0:
            self.alpha += self.fading
            self.bg.set_alpha(self.alpha)
            self.screen.fill((0,0,0))
            if self.alpha <= 10:
                self.fading = 0
                self.bg.set_alpha(0)
                self.fps = 20
                self.after_fadeout()
            self.screen.blit(self.bg, (0,0))
            pygame.display.flip()

    def after_fadeout(self):
        self.changed = 1
    def after_fadein(self):
        self.changed = 1

class Options(BaseState):
    def __init__(self, game):
        BaseState.__init__(self)
        self.bg = game.screen.copy()

        x, y = 4, 3
        bpos = [(x,y)]
        #options button
        button = FlipButton(GFX["btn_options1.png"], GFX["btn_options2.png"],
            callback=self.none)
        self.add_button(button, (x,y), "options")
        x += 65

        #save button
        bpos.append((x,y))
        button = FlipButton(GFX["btn_save1.png"], GFX["btn_save2.png"],
            callback=self.none)
        self.add_button(button, (x,y), "save")
        x += 65

        #load button
        bpos.append((x,y))
        button = FlipButton(GFX["btn_load1.png"], GFX["btn_load2.png"],
            callback=self.none)
        self.add_button(button, (4,y), "load")
        x += 65

        #exit button
        bpos.append((x,y))
        button = FlipButton(GFX["btn_exit1.png"], GFX["btn_exit2.png"],
            callback=self.none)
        self.add_button(button, (4,y), "exit")

    def main_start(self):
        for name in self.button.keys():
            self.button[name].draw(self.bg)
        self.screen.blit(self.bg, (0,0))
        pygame.display.flip()

    def update_other(self):
        pass

    def update_screen(self):
        pass

    def none(self):
        pass
