from pykitten.data import load_object, save_object

from map import Map

class RegionData:
    def __init__(self, reg):
        self.id = reg.id
        self.name = reg.name
        self.owner = reg.owner
        self.pos = reg.pos  #position on map

        self.troops = reg.troops
        self.new_troops = reg.new_troops
        self.build = reg.build
        self.adjacent = reg.adjacent
        self.res = reg.res #(gold, metal, oil) given per turn
        self.pu = reg.pu
        self.hq = reg.hq
        self.town = reg.town

        self.real_pos = reg.real_pos #position on screen
        self.mpos = reg.mpos  #position of map


class PlayerData:
    def __init__(self, p):
        self.color = p.color
        self.ai = p.ai  #artificial intelligence player... aka the computer
        #self.regions must be reconstructed for loading
        self.regnames = p.regnames
        self.res = p.res
        self.cur_res = p.cur_res
        self.res_gain = p.res_gain
        if self.ai:
            self.movements = p.movements

def save_game(game, filename="savegame.dat"):
    #region data
    region = {}
    for id in game.map.region:
        region[id] = RegionData(game.map.region[id])

    #player data
    players = {}
    for id in game.players:
        players[id] = PlayerData(game.players[id])

    #gather and pickle all data
    game_data = {}
    game_data["factions"] = game.factions
    game_data["map_bg"] = game.map.bgname
    game_data["hq"] = game.map.hq
    game_data["regions"] = region
    game_data["players"] = players
    game_data["turn"] = game.turn

    save_object(game_data, filename)


def load_game(filename="savegame.dat"):
    try:
        game_data = load_object(filename)
    except:
        print "Could not load savegame."
        return

    return game_data

