import pygame

from pykitten import utils
from pykitten.data import *
from pykitten.gui import ImgButton, StaticWidget, ProgressBar
from pykitten.utils import Timer

from unitdat import *
from stuff import *

FONT = {}
def load_fonts():
    FONT["small"] = BitMapFont("smallfont.png", 5, 1)

class BitMapFont:
    def __init__(self, filename, width, spacing):
        grid = load_grid(filename, width, width, (0,0,0))[0]
        self.width = width + 1
        self.height = width
        self.chars = {}

        #letters
        for i in xrange(97, 123):
            self.chars[chr(i)] = grid[i-97]

        #numbers
        for i in xrange(10):
            self.chars[str(i)] = grid[i+26]

        #other
        special = [".", ",", "?", "!", "(", ")", "+", "-"]
        for i in xrange(len(special)):
            self.chars[special[i]] = grid[i+36]

    def print_line(self, textline):
        textline = str(textline).lower()
        size = (self.width*(len(textline)), self.height)
        msg = pygame.Surface(size)
        msg.fill((255,0,255))
        msg.set_colorkey((255,0,255))
        x = 0
        for c in textline:
            if c in self.chars.keys():
                if c=="i":
                    x -= 1
                msg.blit(self.chars[c], (x, 0))
                x = x + self.width
                if c=="i":
                    x -= 1
            else:
                x = x + self.width
        return msg



class Fizzle(StaticWidget):
    def __init__(self, images, delay=5):
        size = images[0].get_size()
        StaticWidget.__init__(self, size)
        self.images = images
        self.bg = pygame.Surface(size)
        self.delay = delay
        self.nr = 0
        self.timer = Timer(delay)
        self.active = 0
        self.flip()

    def update(self):
        if self.active and self.timer.update():
            self.flip()
            return True
        return False

    def set_bg(self, img):
        self.bg = img
        self.blit(self.bg, (0,0))
        if self.active:
            self.blit(self.images[self.nr], (0,0))

    def flip(self):
        self.blit(self.bg, (0,0))
        self.blit(self.images[self.nr], (0,0))
        self.nr += 1
        if self.nr >= len(self.images):
            self.nr = 0


class Slider(ImgButton):
    def __init__(self, img, max, cur, psize=(108, 11), ppos=(14,4)):
        ImgButton.__init__(self, img, callback=self.clicked)
        color = (20,200,20)
        bgcolor = (0,0,0)
        self.btn_left = pygame.Rect([(0,0), (11,19)])
        self.btn_right = pygame.Rect([(125,0), (11,19)])
        self.changed = 1
        self.increased = 0
        self.clicked = 0
        self.linked = []
        self.progress = ProgressBar(psize, color, bgcolor)
        self.progress.set_pos(ppos)
        self.poffset = 14
        self.max = max
        self.cur = cur
        self.active = 1
        self.full = False
        self.set_max(max)
        if max > 0:
            self.show_progress()
        else:
            self.progress.change(0.0)
            self.progress.draw(self)
            self.update_links()
            self.active = 0

    def set_active(self, bol):
        if bol and self.max > 0:
            self.active = 1  #re-enable slider
        else:
            self.deactivate()

    def set_full(self, bol):
        self.full = bol

    def deactivate(self):
        self.active = 0
        self.progress.draw(self)

    def add_link(self, link):
        self.linked.append(link)
        self.update_links()
        self.changed = 1

    def show_progress(self):
        self.changed = 1
        if not self.active:
            return
        if self.cur < 0:
            self.cur = 0
        if self.cur > self.max:
            self.cur = self.max
        perc = float(self.cur) / self.max
        self.progress.change(perc)
        self.progress.draw(self)
        self.update_links()

    def set_max(self, max):
        if max==0 and self.cur==0:
            self.deactivate()
        elif not self.changed:
            self.active = 1
            self.changed = 1
            self.max = max
        else:
            self.show_progress()

    def set_cur(self, cur):
        if self.max < cur:
            self.max = cur
        self.cur = cur
        self.show_progress()

    def update_links(self):
        for i in self.linked:
            i.update(self.cur, self.max)

    def set_cur(self, cur):
        self.cur = cur
        self.show_progress()

    def clicked(self):
        if not self.active:
            return
        mpos = pygame.mouse.get_pos()
        pos = self.get_pos()
        ppos = (mpos[0]-pos[0], mpos[1]-pos[1])

        new_cur = self.cur
        if self.progress.rect.collidepoint(ppos):
            new_x = ppos[0]-self.poffset
            perc = float(new_x) / self.progress.length
            new_cur = self.max * perc

            if self.cur==self.max-1 and perc > self.progress.perc and not self.full:
                new_cur = self.max

        elif self.btn_left.collidepoint(ppos) and self.cur > 0:  #left arrow
            new_cur = self.cur - 1
        elif self.btn_right.collidepoint(ppos) and not self.full:  #right arrow
            new_cur = self.cur + 1

        self.clicked = new_cur != self.cur
        self.increased = new_cur > self.cur

        if not self.increased or not self.full:
            if self.cur!= int(new_cur) and not pygame.mixer.get_busy():
                SFX["sldclick"].play()
            self.cur = int(new_cur)


        self.show_progress()


class MoveSlider(Slider):
    def __init__(self, max, cur):
        Slider.__init__(self, GFX["moveslider.png"], max, cur,
            psize=(99,9), ppos=(21,1))
        self.btn_left = pygame.Rect([(0,5), GFX["minus.png"].get_size()])
        self.btn_right = pygame.Rect([(124,0), GFX["plus.png"].get_size()])




class SliderLinkedInfo(StaticWidget):
    def __init__(self, size, side="left"):
        StaticWidget.__init__(self, size)
        self.side = side
        self.bgcolor = (0,0,0)

    def update(self, *args):
        if self.side == "left":
            msg = args[1] - args[0]
        else:
            msg = args[0]

        img = FONT["small"].print_line(msg)
        if self.side == "left":  #align right
            x = self.get_size()[0] - img.get_size()[0]
        else:
            x = 0

        self.fill(self.bgcolor)
        self.blit(img, (x, 0))




class FlipButton(ImgButton):
    def __init__(self, img, img2, callback, args=None, img3=None, \
        img4=None, snd=None):
        ImgButton.__init__(self, img, callback, args)
        self.active = True
        self.image = img
        self.image2 = img2
        self.cur_image = img
        self.pressed = 0
        self.changed = 0
        self.clicksnd = snd

        if img3:
            self.image3 = img3
        else:
            self.image3 = img

        if img4:
            self.image4 = img4
        else:
            self.image4 = img

    def update(self):
        if not self.active:
            return

        mouseover = self.rect.collidepoint(pygame.mouse.get_pos())
        l_button = pygame.mouse.get_pressed()[0]
        if mouseover and l_button and not self.pressed:
            self.pressed = 1
            self.flip(1)

        if mouseover and not l_button and self.pressed:
            self.pressed = 0
            self.flip(0)
            if not pygame.mixer.get_busy() and self.clicksnd:
                self.clicksnd.play()
            if self.arg is None:
                self.callback()
            else:
                self.callback(self.arg)

        if mouseover and not l_button:
            self.hover = True
            self.highlight()

        if not mouseover:
            self.hover = False
            self.pressed = 0
            self.flip(0)

    def flip(self, bol):
        if not bol:
            img = self.image
        else:
            img = self.image2
        if img is not self.cur_image:
            self.changed = 1
        self.cur_image = img
        self.blit(img, (0,0))

    def set_active(self, bol):
        if bol:
            self.activate()
        else:
            self.deactivate()

    def activate(self):
        self.active = True
        self.blit(self.image, (0,0))

    def deactivate(self):
        self.active = False
        self.blit(self.image4, (0,0))


    def highlight(self):
        if self.cur_image is not self.image3 and not self.pressed:
            self.changed = 1
            if not pygame.mixer.get_busy():
                SFX["btnhover"].play()
            self.cur_image = self.image3
            self.blit(self.cur_image, (0,0))


class NamedFlipButton(FlipButton):
    def __init__(self, name, callback, args=None, img4=None):
        img = GFX["btn_"+name+"1.png"]
        img2 = GFX["btn_"+name+"2.png"]
        img3 = GFX["btn_"+name+"3.png"]
        if img4:  #if button has deactivate-image
            img4 = GFX["btn_"+name+"4.png"]
        FlipButton.__init__(self, img, img2, callback, args, img3, img4)
        snd = "btnclick"
        if name=="confirm":
            snd = "confirm"
        elif name=="cancel":
            snd = "cancel"
        self.clicksnd = SFX[snd]


class HitMaskButton(ImgButton):
    def __init__(self, img, callback):
        ImgButton.__init__(self, img, callback=self.clicked)
        self.set_colorkey((255,0,255))
        #self.hitmask = pygame.mask.from_surface(self)
        self.pos = (0,0)

    def set_pos(self, pos):
        """Set widget's position
        """
        self.rect.topleft = pos
        self.pos = pos

    def mouseover(self, mpos):
        p = (mpos[0]-self.pos[0], mpos[1]-self.pos[1])
        return not self.get_at(p)==(255,0,255,255)
        #return self.hitmask.get_at(p)

    def update(self):
        """check if button is clicked or (un-)highlighted
        """
        mpos = pygame.mouse.get_pos()
        if not self.rect.collidepoint(mpos):
            if self.hover:
                self.hover = False
                self.unhighlight()
            return

        mouseover = self.mouseover(mpos)
        l_button = pygame.mouse.get_pressed()[0]

        if mouseover and not self.hover:
            self.hover = True
            self.highlight()
        elif not mouseover and self.hover:
            self.hover = False
            self.unhighlight()

        if mouseover and l_button:
            if self.arg is None:
                self.callback()
            else:
                self.callback(self.arg)


class Container:
    def __init__(self, game):
        self.game = game

    def blit_msg(self, msg, img, pos):
        img.fill((0,0,0))
        msg = str(msg)
        img.blit(FONT["small"].print_line(msg), (0,0))
        self.game.bg.blit(img, pos)

class GameContainer(Container):
    def update_region(self):
        reg = self.game.map.selected
        own = reg.owner==self.game.player.color
        adj = self.game.adjacent(reg, self.game.player.regnames)

        if own or adj:  #show res info
            gold = "+" +  str(reg.res[0])
            metal= "+" +  str(reg.res[1])
            oil = "+" +  str(reg.res[2])
            pu = str(reg.pu)
        else:
            gold, metal, oil = "?", "?", "?"  #hide res info?
            pu = "?"

        x = 52
        y = (460, 485, 510, 535)
        img = pygame.Surface((100,5))
        self.blit_msg(reg.name, img, (x,y[0]))  #name is always shown
        self.blit_msg(gold, img, (x,y[1]))
        self.blit_msg(metal, img, (x,y[2]))
        self.blit_msg(oil, img, (x,y[3]))

        img3 = FONT["small"].print_line("   ")
        if reg.hq:
            img = HEX_img["hq"]
            img2 = GFX["pu.png"]
            self.blit_msg(pu, img3, (118,527))
        elif reg.town:
            img = HEX_img["town"]
            img2 = GFX["pu.png"]
            #img3.blit(FONT["small"].print_line(reg.pu), (0,0))
            self.blit_msg(pu, img3, (118,527))
        else:
            img = pygame.Surface((24,24))
            img.fill((0,0,0))
            img2 = pygame.Surface((12,12))
            img2.fill((0,0,0))
            self.blit_msg("   ", img3, (118,527))
        self.game.bg.blit(img, (108,494))
        self.game.bg.blit(img2, (102,520))

        flag = FLAG[reg.owner+"_s"]
        self.game.bg.blit(flag, (89,555))
        av = False
        avail = reg.get_available()
        for type in TYPES:
            if len(avail[type]) > 0:
                av = True
                break

        self.game.button["move"].set_active(own and reg.occupied() and av)
        self.game.button["build"].set_active(own and (reg.town or reg.hq))
        self.game.button["move"].draw(self.game.bg)
        self.game.button["build"].draw(self.game.bg)
        self.update_troops(own or adj) #only show info for own or adjacent troops

    def update_territories(self):
        count = { "red": 0, "blue": 0, "grey": 0 }
        for name in self.game.map.region.keys():
            reg = self.game.map.region[name]
            count[reg.owner] += 1

        x = 720
        y = (253, 267, 281)
        img = pygame.Surface((45,5))
        self.blit_msg(count["red"], img, (x,y[0]))
        self.blit_msg(count["blue"], img, (x,y[1]))
        self.blit_msg(count["grey"], img, (x,y[2]))

    def update_player(self):
        res = self.game.player.cur_res
        gain = self.game.player.res_gain
        x = 67
        y = 117 #+25
        img = pygame.Surface((70,5))
        for i in xrange(3):
            msg = str(res[i]) + " (+" + str(gain[i]) + ")"
            self.blit_msg(msg, img, (x,y))
            y += 25

        flag = GFX["comm_" + self.game.player.color+".png"].copy()
        self.game.fizzle.set_bg(flag)
        self.game.fizzle.draw(self.game.bg)

    def update_troops(self, show=True):
        reg = self.game.map.selected
        av = reg.get_available()
        dat = []
        if show:
            for type in TYPES:
                msg = str(len(reg.troops[type]))+" ("+str(len(av[type]))+")"
                dat.append(msg)
        else:
            dat = ["?","?","?","?","?"]

        x = (581, 707)
        y = (430, 498, 565)
        img = pygame.Surface((65,5))
        self.blit_msg(dat[0], img, (x[0], y[0]))
        self.blit_msg(dat[1], img, (x[0], y[1]))
        self.blit_msg(dat[2], img, (x[0], y[2]))
        self.blit_msg(dat[3], img, (x[1], y[0]))
        self.blit_msg(dat[4], img, (x[1], y[1]))

    def update(self):
        self.game.map.changed = 1
        self.update_region()
        self.update_player()
        self.update_territories()


class BuildContainer(Container):
    def update_res(self):
        #update player res window
        res = self.game.cur_res
        gain = self.game.res_gain
        x = 67
        y = 117
        img = pygame.Surface((70,5))
        for i in xrange(3):
            msg = str(res[i]) + " (+" + str(gain[i]) + ")"
            self.blit_msg(msg, img, (x,y))
            y += 25

        #update res costs
        pu = self.game.pu - self.game.cur_pu
        cost = self.game.res_cost
        wx, wy = self.game.offset
        pos = [(235,192), (235,208), (296,192)]
        img = pygame.Surface((25,5))
        for i in xrange(3):
            x, y = pos[i]
            self.blit_msg(str(cost[i]), img, (x+wx, y+wy))

        #update pu costs
        self.blit_msg(str(pu), img, (296+wx,208+wy))


    def update_build(self):
        #ordered and max troops info
        wx, wy = self.game.offset
        pos = [(145, 49), (145, 126),  (145, 203), (319, 49), (319,126)]
        troops = self.game.troops
        avail = self.game.avail
        img = MsgLine("", (30,5))
        for i in xrange(len(TYPES)):
            x, y = pos[i]
            img.set_msg(troops[i])
            img.show_msg()
            x1 = x - img.get_size()[0]/2
            self.game.bg.blit(img, (x1+wx, y+wy))

            #img.set_msg("("+str(avail[i])+")")
            img.set_msg(str(avail[i]))
            img.show_msg()
            x2 = x - img.get_size()[0]/2
            self.game.bg.blit(img, (x2+wx, y+wy+22))

            img.set_msg("Max")
            img.show_msg()
            self.game.bg.blit(img, (x2+wx, y+wy+16))

    def update_buttons(self):
        btn = self.game.button

    def update(self):
        self.update_res()
        self.update_build()

class MsgLine(StaticWidget):
    def __init__(self, msg, size, align="c", bgcolor=(0,0,0)):
        StaticWidget.__init__(self, size)
        self.msg = msg
        self.align = align
        self.bgcolor = bgcolor
        self.changed = 0
        self.show_msg()

    def set_msg(self, msg):
        self.msg = msg
        self.changed = 1
        self.show_msg()

    def show_msg(self):
        img = FONT["small"].print_line(self.msg)
        if self.align=="c":
            pos = utils.center(img.get_size(), self.get_size())
        elif self.align=="l":
            pos = (0,0)
        else:
            pos = (self.get_size()[0]-img.get_size()[0], 0)
        self.fill(self.bgcolor)
        self.blit(img, pos)

    def update(self):
        pass

class BitMapTextArea(StaticWidget):
    def __init__(self, width, bgcolor, msg):
        lines = self.gen_lines(msg, width)
        width = width
        height = 7 * len(lines)
        self.bgcolor = bgcolor
        StaticWidget.__init__(self, (width, height))
        self.gen_image(lines)

    def gen_lines(self, text, width):
        length = width / 8 #max number of charas per line
        lines = []
        paras = text.split("\n")
        for p in paras:
            words = p.split(" ")
            line = ""
            for w in words:
                if len(line) < length:
                    line = line + w + " "
                else:
                    lines.append(line)
                    line = w + " "

            if line:  #append last line
                lines.append(line)
        return lines

    def gen_image(self, lines):
        self.fill(self.bgcolor)
        height = 6
        x, y = 1, 1
        for line in lines:
            img = FONT["small"].print_line(line)
            self.blit(img, (x,y))
            y += height
